--[[
	Project: LibItemWidgets
	Version: 1.3
	Author: Zoc ( http://www.riftui.com/forums/member.php?u=1215 )
	
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
]]

if not Library then Library = {} end
if not Library.LibItemWidget then Library.LibItemWidget = {} end

--------------------------------------------------------------------
-- Inventory and Bank Utility Functions

-- Return the size of the specified bag
function Library.LibItemWidget.GetBagSize( bagNumber )
	return Inspect.Item.Detail( Utility.Item.Slot.Inventory( "bag", bagNumber ) )["slots"]
end

-- Return the item in the specified bag and slot.
function Library.LibItemWidget.GetBagItem( bagNumber, slotNumber )
	return Inspect.Item.Detail( Utility.Item.Slot.Inventory( bagNumber, slotNumber ) )
end

function Library.LibItemWidget.GetBankBagSize( bagNumber )
	return Inspect.Item.Detail( Utility.Item.Slot.Bank( "bag", bagNumber ) )["slots"]
end

function Library.LibItemWidget.GetBankBagItem( bagNumber, slotNumber )
	return Inspect.Item.Detail( Utility.Item.Slot.Bank( bagNumber, slotNumber ) )
end

--------------------------------------------------------------------
-- Icon functions

-- Stores the item currently being moved by the cursor
Library.LibItemWidget.carryItemIcon = nil

local iconWidth = 48
local iconHeight = 48

local function SetBagPosition( self, bagNumber, slot )
	self.bagNumber = bagNumber
	self.slot = slot
	self:UpdateIcon()
end

local function UpdateIcon( self )
	self.item = Library.LibItemWidget.GetBagItem( self.bagNumber, self.slot )
	if ( self.item == nil ) then
		-- TODO: Change this.
		self:SetTexture( "Rift", "empty" )
		self:SetBackgroundColor( 0.3, 0.3, 0.3 )
		Library.LibSimpleWidgets.SetBorder( self, 1, 0.5, 0.5, 0.5, 1 )
	else
--		print("Updating Bag #" .. self.bagNumber .. ", Slot #" .. self.slot .. " to " .. self.item["name"])
		self:SetTexture( "Rift", self.item["icon"] )
		if ( self.item["rarity"] == nil ) then
			Library.LibSimpleWidgets.SetBorder( self, 2, 1.0, 1.0, 1.0, 1 )
		elseif ( self.item["rarity"] == "sellable" ) then
			Library.LibSimpleWidgets.SetBorder( self, 2, 0.3, 0.3, 0.3, 1 )
		elseif ( self.item["rarity"] == "uncommon" ) then
			Library.LibSimpleWidgets.SetBorder( self, 2, 0.5, 1.0, 0.5, 1 )
		elseif ( self.item["rarity"] == "rare" ) then
			Library.LibSimpleWidgets.SetBorder( self, 2, 0.3, 0.3, 1.0, 1 )
		elseif ( self.item["rarity"] == "epic" ) then
			Library.LibSimpleWidgets.SetBorder( self, 2, 0.8, 0.3, 1.0, 1 )
		end
	end
end

local function DelayedUpdate( coroutineID, itemwidget )
-- We must handle nil items and regular items.
	if ( itemwidget.item == nil ) then
		while ( itemwidget.item == nil ) do
			Library.LibCoroutine.Pause( 0.2 )
			itemwidget.item = Library.LibItemWidget.GetBagItem( itemwidget.bagNumber, itemwidget.slot )		
		end
	
	else
		local itemid = itemwidget.item["id"]
		while ( ( itemwidget.item ~= nil ) and ( itemid == itemwidget.item["id"] ) ) do
			Library.LibCoroutine.Pause( 0.2 )
			itemwidget.item = Library.LibItemWidget.GetBagItem( itemwidget.bagNumber, itemwidget.slot )
		end
	end
	itemwidget:UpdateIcon()
	Library.LibCoroutine.RemoveCoroutine( coroutineID )
end

function Library.LibItemWidget.Icon( name, parent )
	local itemicon = UI.CreateFrame("Texture", name, parent)
	itemicon:SetWidth( iconWidth )
	itemicon:SetHeight( iconHeight )
	itemicon.SetBagPosition = SetBagPosition
	itemicon.UpdateIcon = UpdateIcon
	itemicon.item = nil
	itemicon.type = "Inventory"
	itemicon.bagNumber = nil
	itemicon.slot = nil

	function itemicon.Event:MouseIn()
		if ( itemicon.item ~= nil ) then
			Command.Tooltip( itemicon.item["id"] )
		end
	end

	function itemicon.Event:MouseOut()
		Command.Tooltip( nil )
	end

	function itemicon.Event:LeftDown()
--	print ( "itemicon.item: " .. tostring( itemicon ) )
		if ( itemicon.item ~= nil ) then
			Command.Cursor( itemicon.item["id"] )
--			print("Bag: " .. itemicon.bagNumber .. " - Item: " .. itemicon.slot )
		end
		Library.LibItemWidget.carryItemIcon = itemicon
	end

	function itemicon.Event:LeftUp()
		Command.Cursor( nil )
--		print("Bag: " .. itemicon.bagNumber .. " - item: " .. itemicon.slot )
--		print("Memory Bag: " .. Library.LibItemWidget.carryItemIcon.bagNumber .. " - item: " .. Library.LibItemWidget.carryItemIcon.slot )
		if (  Library.LibItemWidget.carryItemIcon.item ~= nil ) then
			Command.Item.Move( Utility.Item.Slot.Inventory( Library.LibItemWidget.carryItemIcon.bagNumber, Library.LibItemWidget.carryItemIcon.slot ), Utility.Item.Slot.Inventory( itemicon.bagNumber, itemicon.slot ) )
			if ( ( itemicon.bagNumber ~= Library.LibItemWidget.carryItemIcon.bagNumber ) or ( itemicon.slot ~= Library.LibItemWidget.carryItemIcon.slot ) ) then
				Library.LibCoroutine.RegisterCoroutine( "zBag", DelayedUpdate, Library.LibItemWidget.carryItemIcon )
				Library.LibCoroutine.RegisterCoroutine( "zBag", DelayedUpdate, itemicon )
			end
		end
		Library.LibItemWidget.carryItemIcon = nil
	end
	
	function itemicon.Event:RightClick()
		if ( itemicon.item ~= nil ) then
			print("Add here function to use item: " .. itemicon.item["name"] )
		end
	end

	function itemicon.SetType( icontype )
		itemicon.type = icontype
	end

	return itemicon
end

--------------------------------------------------------------------
-- UI Create Function Overload

local frameConstructors = {
	ItemIcon	= Library.LibItemWidget.Icon,
}

local oldUICreateFrame = UI.CreateFrame
UI.CreateFrame = function(type, name, parent)
	local constructor = frameConstructors[type]
	if constructor then
		return constructor(name, parent)
	else
		return oldUICreateFrame(type, name, parent)
	end
end
